package cz.muni.fi.pb162.project.geometry;

/**
 * Class for creating a snowman from different circular objects.
 *
 * @author Vitalii Bortsov
 */
public class Snowman {
    private final int ballsCount = 4; // in the assignment are written that it should to be public, but maven complains
    private final Circular[] balls = new Circular[ballsCount];
    private final double reduction = 0.8;

    /**
     * Constructor to create a snowman from circular objects.
     *
     * @param sphere lower sphere of snowman
     * @param reductionFactor the upper parts of the snowman will shrink by this factor.
     *                        Must be in range (0..1>. If is not in this range,
     *                        the upper parts of the snowman will shrink by 0.8
     */
    public Snowman(Circular sphere, double reductionFactor) {
        Circular currentBall = sphere;
        Vertex2D currentCenter = sphere.getCenter();
        double reduction = reductionFactor;
        double currentRadius = sphere.getRadius();

        if(reductionFactor <= 0 || reductionFactor > 1) {
            reduction = this.reduction;
        }
        for(int i = 0; i < ballsCount; i++) {
            balls[i] = currentBall;
            currentRadius = currentRadius * reduction;
            currentCenter = new Vertex2D(currentCenter.getX(),
                    currentCenter.getY() + currentBall.getRadius() + currentRadius);
            currentBall = new Circle(currentCenter, currentRadius);
        }
    }

    /**
     * Method returns an array of all "show balls" from the lowest to the highest.
     *
     * @return an array of all "show balls".
     */
    public Circular[] getBalls() {
        return balls;
    }

}
