package cz.muni.fi.pb162.project.geometry;

/**
 * Class for creating a square object.
 *
 * @author Vitalii Bortsov
 */
public class Square implements Circular {
    private final double diameter;
    private final Vertex2D center;
    private final Vertex2D[] arrayOfVertices;

    /**
     * Constructor for creating a square object.
     *
     * @param center coordinates of the center.
     * @param diameter diameter of the circumscribed circle.
     */
    public Square(Vertex2D center, double diameter) {
        this.center = center;
        this.diameter = diameter;
        double radius = diameter / 2;
        arrayOfVertices = new Vertex2D[]
                { new Vertex2D(center.getX() - radius, center.getY()),
                  new Vertex2D(center.getX(), center.getY() - radius),
                  new Vertex2D(center.getX() + radius, center.getY()),
                  new Vertex2D(center.getX(), center.getY() + radius) };
    }

    /**
     * Constructor for creating a square object.
     *
     * @param object circumscribed circle
     */
    public Square(Circular object) {
        this(object.getCenter(), object.getRadius() * 2);
    }

    /**
     *
     * @param index 0 - left vertex, 1 - lower, 2 - right, 3 - upper.
     *              If index is out of range 0-3, returns null.
     * @return vertex on this index or null if index is not in range.
     */
    public Vertex2D getVertex(int index) {
        if(index < 0 || index > 3) {
            return null;
        }
        return arrayOfVertices[index];
    }

    @Override
    public Vertex2D getCenter() {
        return center;
    }

    @Override
    public double getRadius() {
        return diameter / 2;
    }

    @Override
    public String toString() {
        return "Square: vertices=" +
                arrayOfVertices[0] + " " + arrayOfVertices[1] +
                " " + arrayOfVertices[2] + " " + arrayOfVertices[3];
    }
}
